/*
 * Copyright (c) 2011 Imre Fazekas.
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * Neither the name of the Brillien nor the names of its
 * terms and concepts may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.vii.brillien.services.rest;

import com.vii.brillien.core.management.BrillienServices;
import com.vii.brillien.ignition.BrillienContext;
import com.vii.brillien.ignition.transport.BrillienCommunication;
import com.vii.brillien.ignition.transport.BrillienMediator;
import com.vii.brillien.kernel.BrillienException;
import com.vii.brillien.kernel.axiom.sso.Session;
import com.vii.brillien.kernel.axiom.transport.Mediator;
import com.vii.streamline.rest.RestDispatcherServlet;
import com.vii.streamline.services.error.StreamLineException;
import com.vii.streamline.services.reflection.Invocation;
import com.vii.streamline.services.reflection.ReflectionServices;
import com.vii.streamline.structures.collections.InnerList;
import org.apache.xmlrpc.XmlRpcException;
import org.slf4j.Logger;

import javax.servlet.ServletConfig;
import java.util.LinkedList;
import java.util.List;

/**
 * Specialised Servlet class representing a dispatcher providing a RESTful web services interface delegating all incoming calls to the internal business logic.
 */
public class BrillienRestServlet extends RestDispatcherServlet {

    protected static BrillienRestServlet    servletInstance;
    protected BrillienMediator              mediator;

    protected BrillienRestServlet(boolean ssoRequired, Logger logger) {
        super(ssoRequired, logger);
    }

    static {
        servletInstance = new BrillienRestServlet( false, null );
    }

    {
        initMediatorContext();
    }

    protected void initMediatorContext() {

    }

    protected void setupMediatorContext(ServletConfig servletConfig) {

    }

    protected BrillienMediator getMediator() throws StreamLineException {
        if( mediator == null ){
            String username = getServletConfig().getInitParameter("username");
            String password = getServletConfig().getInitParameter("password");

            if( username == null ){
                username = "dispatcher"; password = "dispatcher";
            }

            try {
                if( mediator == null ){
                    setupMediatorContext( getServletConfig() );
                    mediator = BrillienContext.newMediator(username, password);
                    mediator.connect();
                }
            } catch (BrillienException e) {
                throw new StreamLineException( e.getMessage(), e );
            }
        }
        return mediator;
    }

    public boolean isSsoRequired(){
        String sso =  getServletConfig().getInitParameter("ssoRequired");
        boolean ssoRequired = sso == null ? false : Boolean.parseBoolean( sso );

        return  BrillienServices.liaison != null
                ? BrillienServices.liaison.getSSOPresenceName() != null
                : ssoRequired;
    }

    public static BrillienRestServlet getServletInstance() {
        return servletInstance;
    }

    public boolean hasMessageProcessorReturnValue( Invocation invocation ) throws StreamLineException{
        try{
            Boolean res = mediator.sendGet(
                    Session.createSession( invocation.getSessionId() ),
                    invocation.getEntity(),
                    "hasMessageProcessorReturnValue",
                    invocation.getMethod()
            ).acquireResponse();
            return res;
        } catch (BrillienException e) {
            throw new StreamLineException( e );
        }
    }

    public boolean isMethodReckognised(Invocation invocation) {
        return BrillienServices.isServiceExists(invocation);
    }

    @Override
    protected Object call(Invocation invocation) throws StreamLineException {
        if( !isMethodReckognised( invocation ) )
            throw new StreamLineException("Invocation's target cannot be identified.");

        Session session = ssoRequired ? Session.createSession( invocation.getSessionId() ) : null;

        try {
            BrillienMediator mediator = getMediator();

            logger.debug(
                "Calling Presence " +
                invocation.getEntity()
            );

            boolean hasReturnValue = hasMessageProcessorReturnValue( invocation );

            Object payload = null;
            if( hasReturnValue ){
                BrillienCommunication c = mediator.sendGet(
                        session, BrillienContext.RECONNECTION_TIMEOUT,
                        invocation.getEntity(),
                        invocation.getMethod(),
                        invocation.getParameters()
                );
                payload = c.acquireResponse();
            } else {
                mediator.sendSet(
                        session,
                        invocation.getEntity(),
                        invocation.getMethod(),
                        invocation.getParameters()
                );
                payload = "Success.";
            }

            logger.debug(
                "Response received: " + payload
            );

            return payload;
        } catch (Exception e) {
            logger.error( e.getMessage(), e );
            throw new StreamLineException( e.getMessage(), e );
        }
    }

    @Override
    public void destroy() {
        try {
            if (mediator != null)
                mediator.disconnect();
        } catch (BrillienException e) {
        }
        BrillienContext.cleanup();
    }

}
